<?php

namespace Drupal\authorization_code;

use Drupal\authorization_code\Exception\UpsertFailedException;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface;
use Drupal\Core\Password\PasswordInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class AuthorizationCode
 * @package Drupal\authorization_code
 */
class AuthorizationCodeGenerator implements ContainerInjectionInterface {

  /**
   * @var \Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface
   */
  protected $keyValueStore;

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * @var \Drupal\Core\Password\PasswordInterface
   */
  protected $password;

  /**
   * AuthorizationCodeGenerator constructor.
   * @param \Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface $key_value_store
   * @param \Drupal\Core\Config\ImmutableConfig $config
   * @param \Drupal\Core\Password\PasswordInterface $password
   */
  public function __construct(KeyValueStoreExpirableInterface $key_value_store, ImmutableConfig $config, PasswordInterface $password) {
    $this->keyValueStore = $key_value_store;
    $this->config = $config;
    $this->password = $password;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('keyvalue.store.authorization_code'),
      $container->get('config.authorization_code.settings'),
      $container->get('password')
    );
  }


  /**
   * Generates an authorization code for a given key and length.
   *
   * @param string $key
   *   The authorization key to use as an index for the authorization code.
   * @return string
   *   The generated code.
   */
  public function generateCode($key) {
    $length = $this->config->get('code_length');

    $code = $this->doGenerateCode($length);
    $this->writeCode($key, $code);

    return $code;
  }

  /**
   * Generates the code.
   *
   * @param int $length
   *   The length of the code.
   * @return string
   *   The generated code.
   */
  protected function doGenerateCode($length) {
    $max = pow(10, $length);
    $random_int = mt_rand(0, $max - 1);

    // To allow preceding zeros we add $max to the value, convert it to string
    // and remove the first character.
    $value = $max + $random_int;
    $str_value = (string) $value;
    $code = substr($str_value, 1, $length);

    return $code;
  }

  /**
   * @return int
   */
  protected function getCodeExpiration() {
    return $this->config->get('code_expiration_time');
  }

  /**
   * Update or inserts
   * @param string $key
   * @param string $code
   * @throws \Drupal\authorization_code\Exception\UpsertFailedException
   */
  protected function writeCode($key, $code) {
    $this->doWriteCode($key, [
      'hashed_code' => $this->password->hash($code),
      'fetch_count' => 0,
    ]);
  }

  /**
   * @param string $key
   * @param array $code_data
   * @throws \Drupal\authorization_code\Exception\UpsertFailedException
   */
  protected function doWriteCode($key, array $code_data) {
    try {
      $this->keyValueStore
        ->setWithExpire($key, $code_data, $this->getCodeExpiration());
    }
    catch (\Exception $e) {
      throw new UpsertFailedException(
        'Failed to write auth code to the database', $e->getCode(), $e
      );
    }
  }

  /**
   * Verifies that the given code exists in the DB table and the expiration
   * time haven't passed.
   *
   * @param string $key
   *   The authorization key.
   * @param string $code
   *   The authorization code to test.
   * @return bool
   *   true if the code is valid.
   *   false otherwise.
   */
  public function verifyCode($key, $code) {
    $code_data = $this->keyValueStore->get($key, FALSE);
    if (!$code_data) {
      return FALSE;
    }

    $max_fetch_count = $this->config->get('max_fetch_count');
    if ($max_fetch_count > 0 && $code_data['fetch_count'] > $max_fetch_count) {
      return FALSE;
    }

    // Increment the fetch count of the fetched code.
    $code_data['fetch_count']++;
    $this->doWriteCode($key, $code_data);

    return $this->password->check($code, $code_data['hashed_code']);
  }

}
