<?php

namespace Drupal\authorization_code\Plugin\AuthorizationMethod;

use Drupal\authorization_code\Plugin\AuthorizationMethodBase;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Logger\LoggerChannel;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * An authorization method for debug purposes only.
 *
 * When using this method, the authorization code will be logged, but not printed.
 *
 * Examples:
 *  /api/authorization-code/send_code:
 *    {code}
 *    {
 *      "method": "debug",
 *      "argument": "1"
 *    }
 *    {code}
 *  /api/authorization-code/authorize:
 *    {code}
 *    {
 *      "method": "debug",
 *      "argument": "1",
 *      "code": "991781",
 *    }
 *    {code}
 *
 * @AuthorizationMethod(
 *   id = "debug",
 *   label = @Translation("Debug"),
 *   debug_only = true,
 *   argument_type = "integer"
 * )
 */
class DebugAuthorization extends AuthorizationMethodBase {

  /**
   * @var \Drupal\Core\Logger\LoggerChannel
   */
  protected $logger;

  /**
   * DebugAuthorization constructor.
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param \Drupal\Core\TypedData\TypedDataManagerInterface $typed_data_manager
   * @param \Drupal\Core\Logger\LoggerChannel $logger
   * @param \Drupal\Core\Config\ImmutableConfig $settings
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TypedDataManagerInterface $typed_data_manager, LoggerChannel $logger, ImmutableConfig $settings) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $typed_data_manager);
    $this->logger = $logger;
    if (!$settings->get('debug_mode_enabled')) {
      throw new \LogicException('The debug authorization method should not be used outside of debug-mode.');
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('typed_data_manager'),
      $container->get('logger.channel.authorization_code'),
      $container->get('config.authorization_code.settings')
    );
  }

  /**
   * {@inheritdoc}
   */
public function sendCode($argument, $code) {
    $this->logger->debug('The code for @arg is @code', [
      '@arg' => $argument,
      '@code' => $code
    ]);
  }

  public function loadUser($argument) {
    return User::load(1);
  }


}