<?php

namespace Drupal\authorization_code\Plugin\rest\resource;

use Drupal\authorization_code\AuthorizationCodeGenerator;
use Drupal\authorization_code\AuthorizationMethodManager;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Provides a resource for authorizing sessions with a code.
 *
 * @RestResource(
 *   id = "authorization_code:authorize",
 *   label = @Translation("Authorize authorization code"),
 *   uri_paths = {
 *     "create" = "/api/authorization-code/authorize"
 *   }
 * )
 */
class AuthorizeCodeResource extends ResourceBase {

  /**
   * @var \Symfony\Component\HttpFoundation\Session\Session
   */
  protected $session;

  /**
   * @var \Drupal\authorization_code\AuthorizationMethodManager
   */
  protected $manager;

  /**
   * @var \Drupal\authorization_code\AuthorizationCodeGenerator
   */
  protected $generator;

  /**
   * AuthorizeCodeResource constructor.
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param array $serializer_formats
   * @param \Psr\Log\LoggerInterface $logger
   * @param \Symfony\Component\HttpFoundation\Session\Session $session
   * @param \Drupal\authorization_code\AuthorizationMethodManager $manager
   * @param \Drupal\authorization_code\AuthorizationCodeGenerator $generator
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, Session $session, AuthorizationMethodManager $manager, AuthorizationCodeGenerator $generator) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->session = $session;
    $this->manager = $manager;
    $this->generator = $generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('session'),
      $container->get('plugin.manager.authorization_method'),
      $container->get('authorization_code.generator')
    );
  }

  /**
   * The callback method for this resource.
   *
   * @param array $args
   *   An associative array that must contain the following properties:
   *   - method: string
   *       The plugin id
   *   - argument: mixed
   *       The argument.
   *   - code: string
   *       The authorization code.
   * @return \Drupal\rest\ResourceResponse
   */
  public function post(array $args) {
    $method_id = $args['method'];
    $argument = $args['argument'];
    $code = $args['code'];
    $key = implode(':', [$method_id, $argument]);
    if (!$this->generator->verifyCode($key, $code)) {
      throw new BadRequestHttpException('Invalid code provided');
    }

    $method = $this->manager->getInstance(['id' => $method_id]);
    $session_data = ['authorized' => TRUE];
    if ($user = $method->loadUser($argument)) {
      $session_data['user'] = $user;
    }
    $this->session->set('authorization_code', $session_data);
    $this->session->save();
    return new ResourceResponse([
      'message' => $this->t('Authorized')
    ]);
  }

  /**
   * {@inheritdoc}
   */
  protected function getBaseRouteRequirements($method) {
    return array_merge(parent::getBaseRouteRequirements($method), [
      '_csrf_token' => 'TRUE'
    ]);
  }

}
